==> softgun_0.14.orig.tar.gz: softgun-0.14/pca9544.c <==
/*
 * ------------------------------------------------------------------------------
 *
 * Emulation of PCA9544 4-channel I2C-Multiplexer
 *
 * (C) 2004  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 * State: Working, Interrupt-Handling is missing 
 *
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 * ------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include "i2c.h"
#include "signode.h"
#include "pca9544.h"

#if 0
#define dprintf(x...) { fprintf(stderr,x); }
#else
#define dprintf(x...)
#endif


struct PCA9544 {
        I2C_Slave i2c_slave;
	uint8_t ctrl_reg;
	uint8_t new_ctrl_reg;
	SigNode *sda;
	SigNode *scl;
        SigNode *sd[4];
        SigNode *sc[4];
};

/*
 * -----------------------------------------------------
 * Write to the one register of the pca9544
 * Store the value to become valid after stop
 * -----------------------------------------------------
 */
static int
pca9544_write(void *dev,uint8_t data) {
        PCA9544 *pca = dev;
	pca->new_ctrl_reg = data;
        return I2C_ACK;
};

static int 
pca9544_read(void *dev,uint8_t *data)
{
        PCA9544 *pca = dev;
	*data = pca->ctrl_reg;
        return I2C_DONE;
};

static int
pca9544_start(void *dev,int i2c_addr,int operation) {
        dprintf("pca9544 start\n");
        return I2C_ACK;
}

/*
 * -----------------------------------------------
 * Switching is done after the stop condition
 * remove links of incoming signals to old out port 
 * and set a new one
 * -----------------------------------------------
 */
static void
pca9544_stop(void *dev) {
        PCA9544 *pca = dev;
	switch(pca->ctrl_reg & 0x7) {
		case 0:
		case 1:
		case 2:
		case 3:
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			SigNode_RemoveLink(pca->sda,pca->sd[pca->ctrl_reg & 3]);
			SigNode_RemoveLink(pca->scl,pca->sc[pca->ctrl_reg & 3]);
			break;
	}	
	switch(pca->new_ctrl_reg & 0x7) {
		case 0:
		case 1:
		case 2:
		case 3:
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			SigNode_Link(pca->sda,pca->sd[pca->new_ctrl_reg & 3]);
			SigNode_Link(pca->scl,pca->sc[pca->new_ctrl_reg & 3]);
			break;
	}
	pca->ctrl_reg = pca->new_ctrl_reg & 0xf;
        dprintf("pca9544 stop\n");
}

static I2C_SlaveOps pca9544_ops = {
        .start = pca9544_start,
        .stop =  pca9544_stop,
        .read =  pca9544_read,
        .write = pca9544_write
};

I2C_Slave *
PCA9544_New(char *name) {
        PCA9544 *pca = malloc(sizeof(PCA9544));
        I2C_Slave *i2c_slave;
        int i;
        if(!pca) {
                fprintf(stderr,"Out of memory\n");
                exit(8152);
        }
        memset(pca,0,sizeof(PCA9544));
        i2c_slave = &pca->i2c_slave;
        i2c_slave->devops = &pca9544_ops;
        i2c_slave->dev = pca;
	i2c_slave->speed = I2C_SPEED_FAST;
        for(i=0;i<4;i++) {
                pca->sd[i]=SigNode_New("%s.sd%d",name,i);
                if(!pca->sd[i]) {
			exit(1);
                }
                pca->sc[i]=SigNode_New("%s.sc%d",name,i);
                if(!pca->sc[i]) {
			exit(1);
                }
        }
        pca->scl=SigNode_New("%s.scl",name);
        if(!pca->scl) {
		exit(1);
        }
        pca->sda=SigNode_New("%s.sda",name);
        if(!pca->sda) {
		exit(1);
        }
        fprintf(stderr,"PCA9544 I2C-Multiplexer \"%s\" created\n",name);
        return i2c_slave;
}

==> softgun_0.18.orig.tar.gz: softgun-0.18/pca9544.c <==
/*
 * ------------------------------------------------------------------------------
 *
 * Emulation of PCA9544 4-channel I2C-Multiplexer
 *
 * (C) 2004  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 * State: Working, Interrupt-Handling is missing 
 *
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 * ------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include "i2c.h"
#include "signode.h"
#include "sgstring.h"
#include "pca9544.h"

#if 0
#define dbgprintf(x...) { fprintf(stderr,x); }
#else
#define dbgprintf(x...)
#endif


struct PCA9544 {
        I2C_Slave i2c_slave;
	uint8_t ctrl_reg;
	uint8_t new_ctrl_reg;
	SigNode *sda;
	SigNode *scl;
        SigNode *sd[4];
        SigNode *sc[4];
};

/*
 * -----------------------------------------------------
 * Write to the one register of the pca9544
 * Store the value to become valid after stop
 * -----------------------------------------------------
 */
static int
pca9544_write(void *dev,uint8_t data) {
        PCA9544 *pca = dev;
	pca->new_ctrl_reg = data;
        return I2C_ACK;
};

static int 
pca9544_read(void *dev,uint8_t *data)
{
        PCA9544 *pca = dev;
	*data = pca->ctrl_reg;
        return I2C_DONE;
};

static int
pca9544_start(void *dev,int i2c_addr,int operation) {
        dbgprintf("pca9544 start\n");
        return I2C_ACK;
}

/*
 * -----------------------------------------------
 * Switching is done after the stop condition
 * remove links of incoming signals to old out port 
 * and set a new one
 * -----------------------------------------------
 */
static void
pca9544_stop(void *dev) {
        PCA9544 *pca = dev;
	switch(pca->ctrl_reg & 0x7) {
		case 0:
		case 1:
		case 2:
		case 3:
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			SigNode_RemoveLink(pca->sda,pca->sd[pca->ctrl_reg & 3]);
			SigNode_RemoveLink(pca->scl,pca->sc[pca->ctrl_reg & 3]);
			break;
	}	
	switch(pca->new_ctrl_reg & 0x7) {
		case 0:
		case 1:
		case 2:
		case 3:
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			SigNode_Link(pca->sda,pca->sd[pca->new_ctrl_reg & 3]);
			SigNode_Link(pca->scl,pca->sc[pca->new_ctrl_reg & 3]);
			break;
	}
	pca->ctrl_reg = pca->new_ctrl_reg & 0xf;
        dbgprintf("pca9544 stop\n");
}

static I2C_SlaveOps pca9544_ops = {
        .start = pca9544_start,
        .stop =  pca9544_stop,
        .read =  pca9544_read,
        .write = pca9544_write
};

I2C_Slave *
PCA9544_New(char *name) {
        PCA9544 *pca = sg_new(PCA9544);
        I2C_Slave *i2c_slave;
        int i;
        i2c_slave = &pca->i2c_slave;
        i2c_slave->devops = &pca9544_ops;
        i2c_slave->dev = pca;
	i2c_slave->speed = I2C_SPEED_FAST;
        for(i=0;i<4;i++) {
                pca->sd[i]=SigNode_New("%s.sd%d",name,i);
                if(!pca->sd[i]) {
			exit(1);
                }
                pca->sc[i]=SigNode_New("%s.sc%d",name,i);
                if(!pca->sc[i]) {
			exit(1);
                }
        }
        pca->scl=SigNode_New("%s.scl",name);
        if(!pca->scl) {
		exit(1);
        }
        pca->sda=SigNode_New("%s.sda",name);
        if(!pca->sda) {
		exit(1);
        }
        fprintf(stderr,"PCA9544 I2C-Multiplexer \"%s\" created\n",name);
        return i2c_slave;
}
